﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace WebSiteAdminTool.UI.Views.CustomControls
{
    [TemplateVisualState(Name = "Default", GroupName = "WatermarkStates"),
    TemplateVisualState(Name = "Selected", GroupName = "WatermarkStates")]
    public class StateHyperLink : HyperlinkButton
    {
        #region enums

        public enum HyperLinkStates
        {
            Default,
            Selected
        }

        #endregion

        #region custom properties

        /// <summary>
        /// Watermark text dependency property
        /// </summary>
        public static readonly DependencyProperty CurrentStateProperty = DependencyProperty.Register(
            "CurrentState",
            typeof(HyperLinkStates),
            typeof(StateHyperLink),
            new PropertyMetadata("Choose State"));

        /// <summary>
        /// Gets or sets the watermark.
        /// </summary>
        /// <value>The watermark.</value>
        [Description("Gets or sets the state")]
        [Category("CurrentState")]
        public HyperLinkStates CurrentState
        {
            get { return (HyperLinkStates)GetValue(CurrentStateProperty); }
            set { SetValue(CurrentStateProperty, value); }
        }

        #endregion

        #region ctors

        /// <summary>
        /// Initializes a new instance of the <see cref="WatermarkedTextBox"/> class.
        /// </summary>
        public StateHyperLink()
        {
            DefaultStyleKey = typeof(HyperlinkButton);
        }

        #endregion


    }
}
